<?php

namespace app\models;

use Anper\Iuliia\Iuliia;
use Yii;
use yii\helpers\FileHelper;
use yii\web\BadRequestHttpException;
use yii\web\UploadedFile;

/**
 * This is the model class for table "image".
 *
 * @property int $id
 * @property string $sha256
 * @property string $original_name
 */
class Image extends \yii\db\ActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'image';
    }

    public static function fromUploadedFile(?UploadedFile $file): ?Image
    {
        if ($file === null) {
            return null;
        }

        $hash = hash_file('sha256', $file->tempName);
        $existing = Image::find()->where([ 'sha256' => $hash ])->one();
        if ($existing) {
            return $existing;
        }

        copy($file->tempName, env('UPLOADS_PATH') . '/' . $hash);

        if (!preg_match('/^[a-z0-9-_]+(|\.[\w\d]+)$/i', Iuliia::translate($file->name, Iuliia::WIKIPEDIA))) {
            throw new BadRequestHttpException('Filename must be a-Z0-9_-!');
        }

        $new = new Image([
            'sha256' => $hash,
            'original_name' => Iuliia::translate($file->name, Iuliia::WIKIPEDIA),
            'mime' => FileHelper::getMimeType($file->tempName)
        ]);
        $new->save(true);

        return $new;
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['sha256', 'original_name'], 'required'],
            [['original_name'], 'match', 'pattern' => '/^[a-z0-9-_]+(|\.[\w\d]+)$/i'],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'sha256' => 'Sha256',
            'original_name' => 'Original Name',
        ];
    }
}
